// A pucBank holds various arrays of values which can be accessed by a key.
// Some of these values are accessable by the end user, some are not
function pucBank() {

  this.name = "";  

  // An array of strings
  this.keys = new Array();
  
  // An array of arrays of strings, the index of which correlates with the
  // index of its key
  this.values = new Array();

  // An array of arrays of bools, saying whether the value was selected
  this.selected = new Array();
  
  // An array of bools that determines whether the user can change a key type
  this.userVisible = new Array();

  // An array that says how the particular key can be selected by the user, as
  // XUL's listbox does it:
  // "single" - Only one row may be selected
  // "multiple" - Multiple rows may be selected
  this.selType = new Array();

  this.copyInitialize = function(other) {
    this.keys = new Array();
    this.values = new Array();
    this.selected = new Array();
    this.userVisible = new Array();
    this.name = other.name;
    this.selType = new Array();
    
    var i;
    var j;
    for(i = 0; i < other.keys.length; i++) {
      this.keys.push(other.keys[i]);
      this.values.push(new Array());
      this.selected.push(new Array());
      this.userVisible.push(other.userVisible[i]);
      this.selType.push(other.selType[i]);

      for(j = 0; j < other.values[i].length; j++) {
        this.values[i].push(other.values[i][j]);
        this.selected[i].push(other.selected[i][j]);
      }
    }
    return this;
  }

  // Access functions
  this.getValuesFromKey = function(key) {
    for(var i = 0; i < this.keys.length; i++) {
      if(this.keys[i] == key)
        return this.values[i];
    }
    return null;
  }

  // Note this is by reference
  this.setValuesFromKey = function(key, newValueArray) {
    for(var i = 0; i < this.keys.length; i++) {
      if(this.keys[i] == key)
        return (this.values[i] = newValueArray);
    }
    return null;
  }

  // Returns an array corresponding the the values array
  // Saying whether values are selected
  this.getSelectedFromKey = function(key) {
    for(var i = 0; i < this.keys.length; i++) {
      if(this.keys[i] == key)
        return this.selected[i];
    }
    return null;
  }

  this.setSelectedFromKey = function(key, newSelectedArray) {
    for(var i = 0; i < this.keys.length; i++) {
      if(this.keys[i] == key)
        return (this.seleted[i] = newSelectedArray);
    }
    return null;
  }

  this.addKey = function(key, userVisible, selType) {
    this.keys.push(key);
    this.values.push(new Array());
    this.selected.push(new Array());
    this.userVisible.push(userVisible);
    this.selType.push(selType);
  }

  this.removeKey = function(key) {
    for(var i = 0; i < this.keys.length; i++) {
      if(this.keys[i] == key) {
        this.values.splice(i, 1);
        this.selected.splice(i, 1);
        this.userVisible.splice(i, 1);
        this.selType.splice(i, 1);
        var spliced = this.keys.splice(i, 1);
        return spliced[0];
      }
    }
    return null;
  }

  this.isKeyUserVisible = function(key) {
    for(var i = 0; i < this.keys.length; i++) {
      if(this.keys[i] == key)
        return this.userVisible[i];
    }
    return null;
  }

  this.getKeySelType = function(key) {
    for(var i = 0; i < this.keys.length; i++) {
      if(this.keys[i] == key)
        return this.selType[i];
    }
    return null;
  }


  // XML Conversion

  // Sample XML:
  // <bank name="Filter Default">
  //   <key name="filter" userVisible="false" selType="multiple"> 
  //     <value selected="true">Filter Contents</value>
  //   </key>
  // </bank>

  // Initializes this object from an XML node, returns the name
  this.XMLInitialize = function(node) {
    if(node.getAttributeNode("name"))
      this.name = node.getAttributeNode("name").value;
    var keys = 0;
    var keyNode = null;
    for(; (keyNode = node.childNodes[keys]); keys++) {
      if(keyNode.nodeName == "key") {
        this.keys.push(keyNode.getAttributeNode("name").value);
        this.userVisible.push( (keyNode.getAttributeNode("userVisible").value == "true"));
        this.selType.push( (keyNode.getAttributeNode("selType").value));
        this.values.push(new Array());
        this.selected.push(new Array());
  
        var vals = 0;
        var valNode = null;
        for(; (valNode = keyNode.childNodes[vals]); vals++) {
          if(valNode.nodeName == "value") {
           if(valNode.firstChild)
             this.values[this.values.length-1].push(valNode.firstChild.nodeValue);
           else
             this.values[this.values.length-1].push("");
           this.selected[this.selected.length-1].push(valNode.getAttributeNode("selected").value == "true");
          }
        }
      }
    }
    return this;
  }

  this.toXMLNode = function(doc) {
    var bankNode = doc.createElement("bank");
    bankNode.setAttribute("name", this.name);
    
    for(var i = 0; i < this.keys.length; i++) {
      bankNode.appendChild(doc.createTextNode("\n\t\t\t"));
      var keyNode = doc.createElement("key");

      keyNode.setAttribute("name", this.keys[i]);
      keyNode.setAttribute("userVisible", this.userVisible[i]);
      keyNode.setAttribute("selType", this.selType[i]);

      for(var j = 0; j < this.values[i].length; j++) {
        keyNode.appendChild(doc.createTextNode("\n\t\t\t\t"));
        var valueNode = doc.createElement("value");
        valueNode.appendChild(doc.createTextNode(this.values[i][j]));
        valueNode.setAttribute("selected", this.selected[i][j])
        keyNode.appendChild(valueNode);
      }
      keyNode.appendChild(doc.createTextNode("\n\t\t\t"));
      bankNode.appendChild(keyNode);      
    }
    bankNode.appendChild(doc.createTextNode("\n\t\t"));
    return bankNode;
  }


  // Allows bank to be editted by the user
  // bool development determines whether to show the dialog
  // in development mode.  
  // Returns true if the user clicked "OK", false if they cancelled
  this.doUserEdit = function(checker, development) {
    var params = {checker: checker, development: development, bank: this, newBank: null};
    window.openDialog("chrome://puc/content/pucBank.xul", "pubBank", "centerscreen,modal", params);
    if(this != params.newBank) {
      this.copyInitialize(params.newBank);
      return true;
    } else {
      return false;
    }
  }
}