// This describes the "class" pucPage

function pucPage() {

  // Initialization

  this.id = null;
  this.title = "";
  this.url = null;

  // Default user accessible settings:
  this.enabled = false;
  this.period = 5;
  this.units = "minutes";

  this.checker = null;  // It is the initializer's responsibility to find a default checker

  // This is to be used only for the XML loader to assign a checker
  // this this.checker based on the page's "checker" attribute (which is the 
  // name of the checker).
  this.checkerXMLPointer = "";  

  //An array of checker Banks, built up as the user uses various checkers
  this.checkerBanks = new Array();  

  this.checkAtStartup = false;

  this.timer = null;
  this.hasBeenSaved = false;

  // These are only set after a check for updates, they should be 
  // considered valid only during an update check
  this.oldSource = "";
  this.newSource = "";

  // Constructors

  // Creates a deep copy of a page.
  // Watch out, this.checker is still a reference
  this.copyInitialize = function(other) {
    with(this) {
      id = other.id;
      title = other.title;
      url = other.url;
      enabled = other.enabled;
      period = other.period;
      units = other.units;
      checkAtStartup = other.checkAtStartup;         
      this.timer = other.timer;
      this.checker = other.checker;

      for(var i = 0; i < other.checkerBanks.length; i++) {
        var copyBank = new pucBank();
        this.checkerBanks.push(copyBank.copyInitialize(other.checkerBanks[i]));
      }
      hasBeenSaved = other.hasBeenSaved;
      oldSource = other.oldSource;
      newSource = other.newSource;
    }
    return this;
  }

  // Initializes this object from an XML node
  this.XMLInitialize = function(pageNode) {
    
    if(pageNode.getAttributeNode("id"))
      this.id = pageNode.getAttributeNode("id").value;

    this.title = pageNode.getAttributeNode("title").value;
    this.url = pageNode.getAttributeNode("url").value;
    this.period = pageNode.getAttributeNode("period").value;
    this.enabled = (pageNode.getAttribute("enabled") == "true");
    this.units = pageNode.getAttributeNode("units").value;
    this.checkAtStartup = (pageNode.getAttribute("checkAtStartup") == "true");

    if(pageNode.getAttributeNode("checker"))
      this.checkerXMLPointer = pageNode.getAttributeNode("checker").value;

    // We can assume that the page was downloaded and saved, at a minimum, 
    // when it was first loaded into the cinfiguration, and its cache is
    // still available.
    this.hasBeenSaved = true;

    var subNodes = 0;
    var subNode = null;
    for(; (subNode = pageNode.childNodes[subNodes]); subNodes++) {
      if(subNode.nodeName == "bank") {
        var newBank = new pucBank();
        this.checkerBanks.push(newBank.XMLInitialize(subNode));
      }
    }

    // Todo: Recurse through each page's checkers
    return this;
  }

  //Methods:

  // if saveID is true, the XML node will contain the ID.
  // When we are exporting, we don't want to include the ID.
  this.toXMLNode = function(doc, saveID) {
    var pageNode = doc.createElement("page");

    if(saveID)  
      pageNode.setAttribute("id", this.id);

    pageNode.setAttribute("title", this.title);
    pageNode.setAttribute("url", this.url);
    pageNode.setAttribute("enabled", this.enabled);
    pageNode.setAttribute("period", this.period);
    pageNode.setAttribute("units", this.units);
    pageNode.setAttribute("checkAtStartup", this.checkAtStartup);
    if(this.checker)
      pageNode.setAttribute("checker", this.checker.name);

    // Save all databanks
    for(var i=0; i < this.checkerBanks.length; i++) {
      pageNode.appendChild(doc.createTextNode("\n\t\t"));
      pageNode.appendChild(this.checkerBanks[i].toXMLNode(doc));
    }
    pageNode.appendChild(doc.createTextNode("\n\t"));
    return pageNode;
  }

  this.saveSource = function(source) {
    var foStream = puc_getWriteStream(this.id);
    
    // We have to convert unicode so that it can be correctly saved
    // Note: In Firefox 1.0.*, UTF-16 and 32 cannot be read properly.
    var conSource = puc_fromUnicode(source, "UTF-8");
    foStream.write(conSource, conSource.length);
    
    // This was the original method of saving up through version 0.3:
    //foStream.write(source, source.length);
    foStream.close();
  }

  // "self" is used as an alias for "this" because of the embedded functions

  // Gets the previously saved source code asynchronously, saves it to
  // this.oldSource, then calls onCompletion(self)
  this.getOldSource = function(self, onCompletion) {
    var file = puc_getFile(self.id);
    //var file = puc_getFile("utftest");
    self.oldSource = "";
    var ios = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
    var fileURI = ios.newFileURI(file);
    var channel = ios.newChannelFromURI(fileURI);
    var observer = {
      onStreamComplete : function(aLoader, aContext, aStatus, aLength, oldText) {
        try {
          // puc_toUnicode (i believe) will throw an error when loading the cache
	  // saved in versions 0.2 and earlier
          self.oldSource = puc_toUnicode(oldText, "UTF-8");
	  if(!self.oldSource) 
            throw(" ");
	} catch (e) {
          puc_logMessage("Reverting to version 0.2 page loading (without unicode).");
	  self.oldSource = oldText;
	}
	onCompletion(self);
      }
    }
    var sl = Components.classes["@mozilla.org/network/stream-loader;1"]
      .createInstance(Components.interfaces.nsIStreamLoader);
    sl.init(channel, observer, null);
  }

  // Downloads the newest source code, saves it to this.newSource, then calls onCompletion(self, failed),
  // where failed is true if there was an error downloading
  this.getNewSource = function(self, onCompletion) {
    // We use XMLHttpRequest which can actually get us the raw "text" for 
    // anything, including pages, pictures, etc.
    var req = new XMLHttpRequest();

    // We have to embed the funciton so that it has easy access to our page
    // This avoids problems with simultaneous update checks while still 
    // downloading in the background, not holding up the UI
    req.onreadystatechange = function ()
    {
      if (req.readyState != 4) {
        // Do Nothing -- We haven't finished downloading
      } else {
        // Save the downloaded source
        self.newSource = req.responseText;
        onCompletion(self, req.status != 200);
      }
    }
    puc_logMessage("Downloading id " + self.id + ": " + self.url + " for update check");
    req.open("GET", self.url, true);
    req.send(null);
  }

  // Compare this page's new and old html, if no change, it calls onNoChange(this),
  // it calles onChange(this) if there was a change in the source
  this.testForUpdate = function(onNoChange, onChange) {	
    //puc_logMessage("Called to test for update on id: " + this.id + " - " + this.title);

    // Once getOldSource opens the page, it calls the embedded function here
    this.getOldSource(this, function(self) {
      self.getNewSource(self, function(self, failed) {
        if(failed) {
          puc_logMessage("There was a problem downloading the webpage " + self.url + " to compare.");
          if(PrefUtils.getPref("puc.alertonfailure")) {
            alert("Page Update Checker: There was a problem downloading the webpage " + self.url + " to compare.\n"+
              "To remove this message, go to Preferences in the Page Update Checker extension options");
          }

        // If we've never saved this file, then just save it and don't do a comparrison
        } else if(!self.hasBeenSaved) {
          // Simply write the file
          puc_logMessage("Saving html from id " + self.id + " " + self.url + " for the first time");
          self.saveSource(self.newSource);
          self.hasBeenSaved = true;
        } else {     
          self.saveSource(self.newSource); 
          if(self.hasPageChanged())
            onChange(self);
          else
            onNoChange(self);
        }
      });
    });
  }

  // Returns true if the page has changed, false if not
  this.hasPageChanged = function () {
    return this.checker.checkForUpdates(this);
  }

  // Get a bank based on its name
  this.getCheckerBankByName = function(name) {
    for(var i = 0; i < this.checkerBanks.length; i++) {
      if(this.checkerBanks[i].name == name)
        return this.checkerBanks[i];
    }
    return null;
  }

  // Get the bank belonging to the current checker, if one doesn't
  // exist, it copies the default checker into it's own array of banks
  this.getBank = function() {

    if(this.checker == null)
      return null;

    var bank = this.getCheckerBankByName(this.checker.name);
    if(bank == null) {
      bank = new pucBank();
      bank.copyInitialize(this.checker.defaultBank);
      bank.name = this.checker.name;
      this.checkerBanks.push(bank);
    }
    return bank;
  }
}
